<?php

// autoload_classmap.php @generated by Composer

$vendorDir = dirname(dirname(__FILE__));
$baseDir = dirname($vendorDir);

return array(
    'mageekguy\\atoum\\adapter' => $vendorDir . '/atoum/atoum/classes/adapter.php',
    'mageekguy\\atoum\\adapter\\definition' => $vendorDir . '/atoum/atoum/classes/adapter/definition.php',
    'mageekguy\\atoum\\annotations\\extractor' => $vendorDir . '/atoum/atoum/classes/annotations/extractor.php',
    'mageekguy\\atoum\\asserter' => $vendorDir . '/atoum/atoum/classes/asserter.php',
    'mageekguy\\atoum\\asserter\\definition' => $vendorDir . '/atoum/atoum/classes/asserter/definition.php',
    'mageekguy\\atoum\\asserter\\exception' => $vendorDir . '/atoum/atoum/classes/asserter/exception.php',
    'mageekguy\\atoum\\asserter\\generator' => $vendorDir . '/atoum/atoum/classes/asserter/generator.php',
    'mageekguy\\atoum\\asserter\\resolver' => $vendorDir . '/atoum/atoum/classes/asserter/resolver.php',
    'mageekguy\\atoum\\asserters\\adapter' => $vendorDir . '/atoum/atoum/classes/asserters/adapter.php',
    'mageekguy\\atoum\\asserters\\adapter\\call' => $vendorDir . '/atoum/atoum/classes/asserters/adapter/call.php',
    'mageekguy\\atoum\\asserters\\adapter\\call\\exceptions\\logic' => $vendorDir . '/atoum/atoum/classes/asserters/adapter/call/exceptions/logic.php',
    'mageekguy\\atoum\\asserters\\adapter\\call\\manager' => $vendorDir . '/atoum/atoum/classes/asserters/adapter/call/manager.php',
    'mageekguy\\atoum\\asserters\\adapter\\call\\manager\\exception' => $vendorDir . '/atoum/atoum/classes/asserters/adapter/call/manager/exception.php',
    'mageekguy\\atoum\\asserters\\adapter\\exceptions\\logic' => $vendorDir . '/atoum/atoum/classes/asserters/adapter/exceptions/logic.php',
    'mageekguy\\atoum\\asserters\\boolean' => $vendorDir . '/atoum/atoum/classes/asserters/boolean.php',
    'mageekguy\\atoum\\asserters\\castToArray' => $vendorDir . '/atoum/atoum/classes/asserters/castToArray.php',
    'mageekguy\\atoum\\asserters\\castToString' => $vendorDir . '/atoum/atoum/classes/asserters/castToString.php',
    'mageekguy\\atoum\\asserters\\constant' => $vendorDir . '/atoum/atoum/classes/asserters/constant.php',
    'mageekguy\\atoum\\asserters\\dateInterval' => $vendorDir . '/atoum/atoum/classes/asserters/dateInterval.php',
    'mageekguy\\atoum\\asserters\\dateTime' => $vendorDir . '/atoum/atoum/classes/asserters/dateTime.php',
    'mageekguy\\atoum\\asserters\\error' => $vendorDir . '/atoum/atoum/classes/asserters/error.php',
    'mageekguy\\atoum\\asserters\\exception' => $vendorDir . '/atoum/atoum/classes/asserters/exception.php',
    'mageekguy\\atoum\\asserters\\extension' => $vendorDir . '/atoum/atoum/classes/asserters/extension.php',
    'mageekguy\\atoum\\asserters\\hash' => $vendorDir . '/atoum/atoum/classes/asserters/hash.php',
    'mageekguy\\atoum\\asserters\\integer' => $vendorDir . '/atoum/atoum/classes/asserters/integer.php',
    'mageekguy\\atoum\\asserters\\iterator' => $vendorDir . '/atoum/atoum/classes/asserters/iterator.php',
    'mageekguy\\atoum\\asserters\\mock' => $vendorDir . '/atoum/atoum/classes/asserters/mock.php',
    'mageekguy\\atoum\\asserters\\mock\\exceptions\\logic' => $vendorDir . '/atoum/atoum/classes/asserters/mock/exceptions/logic.php',
    'mageekguy\\atoum\\asserters\\mysqlDateTime' => $vendorDir . '/atoum/atoum/classes/asserters/mysqlDateTime.php',
    'mageekguy\\atoum\\asserters\\object' => $vendorDir . '/atoum/atoum/classes/asserters/object.php',
    'mageekguy\\atoum\\asserters\\output' => $vendorDir . '/atoum/atoum/classes/asserters/output.php',
    'mageekguy\\atoum\\asserters\\phpArray' => $vendorDir . '/atoum/atoum/classes/asserters/phpArray.php',
    'mageekguy\\atoum\\asserters\\phpArray\\child' => $vendorDir . '/atoum/atoum/classes/asserters/phpArray/child.php',
    'mageekguy\\atoum\\asserters\\phpClass' => $vendorDir . '/atoum/atoum/classes/asserters/phpClass.php',
    'mageekguy\\atoum\\asserters\\phpFloat' => $vendorDir . '/atoum/atoum/classes/asserters/phpFloat.php',
    'mageekguy\\atoum\\asserters\\phpFunction' => $vendorDir . '/atoum/atoum/classes/asserters/phpFunction.php',
    'mageekguy\\atoum\\asserters\\phpResource' => $vendorDir . '/atoum/atoum/classes/asserters/phpResource.php',
    'mageekguy\\atoum\\asserters\\phpString' => $vendorDir . '/atoum/atoum/classes/asserters/phpString.php',
    'mageekguy\\atoum\\asserters\\sizeOf' => $vendorDir . '/atoum/atoum/classes/asserters/sizeOf.php',
    'mageekguy\\atoum\\asserters\\stream' => $vendorDir . '/atoum/atoum/classes/asserters/stream.php',
    'mageekguy\\atoum\\asserters\\testedClass' => $vendorDir . '/atoum/atoum/classes/asserters/testedClass.php',
    'mageekguy\\atoum\\asserters\\utf8String' => $vendorDir . '/atoum/atoum/classes/asserters/utf8String.php',
    'mageekguy\\atoum\\asserters\\variable' => $vendorDir . '/atoum/atoum/classes/asserters/variable.php',
    'mageekguy\\atoum\\autoloader' => $vendorDir . '/atoum/atoum/classes/autoloader.php',
    'mageekguy\\atoum\\autoloader\\mock' => $vendorDir . '/atoum/atoum/classes/autoloader/mock.php',
    'mageekguy\\atoum\\cli' => $vendorDir . '/atoum/atoum/classes/cli.php',
    'mageekguy\\atoum\\cli\\clear' => $vendorDir . '/atoum/atoum/classes/cli/clear.php',
    'mageekguy\\atoum\\cli\\colorizer' => $vendorDir . '/atoum/atoum/classes/cli/colorizer.php',
    'mageekguy\\atoum\\cli\\command' => $vendorDir . '/atoum/atoum/classes/cli/command.php',
    'mageekguy\\atoum\\cli\\command\\exception' => $vendorDir . '/atoum/atoum/classes/cli/command/exception.php',
    'mageekguy\\atoum\\cli\\commands\\git' => $vendorDir . '/atoum/atoum/classes/cli/commands/git.php',
    'mageekguy\\atoum\\cli\\progressBar' => $vendorDir . '/atoum/atoum/classes/cli/progressBar.php',
    'mageekguy\\atoum\\cli\\prompt' => $vendorDir . '/atoum/atoum/classes/cli/prompt.php',
    'mageekguy\\atoum\\configurator' => $vendorDir . '/atoum/atoum/classes/configurator.php',
    'mageekguy\\atoum\\exception' => $vendorDir . '/atoum/atoum/classes/exception.php',
    'mageekguy\\atoum\\exceptions\\logic' => $vendorDir . '/atoum/atoum/classes/exceptions/logic.php',
    'mageekguy\\atoum\\exceptions\\logic\\badMethodCall' => $vendorDir . '/atoum/atoum/classes/exceptions/logic/badMethodCall.php',
    'mageekguy\\atoum\\exceptions\\logic\\invalidArgument' => $vendorDir . '/atoum/atoum/classes/exceptions/logic/invalidArgument.php',
    'mageekguy\\atoum\\exceptions\\runtime' => $vendorDir . '/atoum/atoum/classes/exceptions/runtime.php',
    'mageekguy\\atoum\\exceptions\\runtime\\file' => $vendorDir . '/atoum/atoum/classes/exceptions/runtime/file.php',
    'mageekguy\\atoum\\exceptions\\runtime\\unexpectedValue' => $vendorDir . '/atoum/atoum/classes/exceptions/runtime/unexpectedValue.php',
    'mageekguy\\atoum\\extension' => $vendorDir . '/atoum/atoum/classes/extension.php',
    'mageekguy\\atoum\\extension\\configuration' => $vendorDir . '/atoum/atoum/classes/extension/configuration.php',
    'mageekguy\\atoum\\factory\\builder' => $vendorDir . '/atoum/atoum/classes/factory/builder.php',
    'mageekguy\\atoum\\factory\\builder\\closure' => $vendorDir . '/atoum/atoum/classes/factory/builder/closure.php',
    'mageekguy\\atoum\\fs\\path' => $vendorDir . '/atoum/atoum/classes/fs/path.php',
    'mageekguy\\atoum\\fs\\path\\exception' => $vendorDir . '/atoum/atoum/classes/fs/path/exception.php',
    'mageekguy\\atoum\\fs\\path\\factory' => $vendorDir . '/atoum/atoum/classes/fs/path/factory.php',
    'mageekguy\\atoum\\includer' => $vendorDir . '/atoum/atoum/classes/includer.php',
    'mageekguy\\atoum\\includer\\exception' => $vendorDir . '/atoum/atoum/classes/includer/exception.php',
    'mageekguy\\atoum\\iterators\\filters\\recursives\\atoum\\source' => $vendorDir . '/atoum/atoum/classes/iterators/filters/recursives/atoum/source.php',
    'mageekguy\\atoum\\iterators\\filters\\recursives\\closure' => $vendorDir . '/atoum/atoum/classes/iterators/filters/recursives/closure.php',
    'mageekguy\\atoum\\iterators\\filters\\recursives\\dot' => $vendorDir . '/atoum/atoum/classes/iterators/filters/recursives/dot.php',
    'mageekguy\\atoum\\iterators\\filters\\recursives\\extension' => $vendorDir . '/atoum/atoum/classes/iterators/filters/recursives/extension.php',
    'mageekguy\\atoum\\iterators\\recursives\\atoum\\source' => $vendorDir . '/atoum/atoum/classes/iterators/recursives/atoum/source.php',
    'mageekguy\\atoum\\iterators\\recursives\\directory\\factory' => $vendorDir . '/atoum/atoum/classes/iterators/recursives/directory/factory.php',
    'mageekguy\\atoum\\locale' => $vendorDir . '/atoum/atoum/classes/locale.php',
    'mageekguy\\atoum\\mailer' => $vendorDir . '/atoum/atoum/classes/mailer.php',
    'mageekguy\\atoum\\mailers\\mail' => $vendorDir . '/atoum/atoum/classes/mailers/mail.php',
    'mageekguy\\atoum\\mock\\aggregator' => $vendorDir . '/atoum/atoum/classes/mock/aggregator.php',
    'mageekguy\\atoum\\mock\\controller' => $vendorDir . '/atoum/atoum/classes/mock/controller.php',
    'mageekguy\\atoum\\mock\\controller\\invoker' => $vendorDir . '/atoum/atoum/classes/mock/controller/invoker.php',
    'mageekguy\\atoum\\mock\\controller\\iterator' => $vendorDir . '/atoum/atoum/classes/mock/controller/iterator.php',
    'mageekguy\\atoum\\mock\\controller\\linker' => $vendorDir . '/atoum/atoum/classes/mock/controller/linker.php',
    'mageekguy\\atoum\\mock\\generator' => $vendorDir . '/atoum/atoum/classes/mock/generator.php',
    'mageekguy\\atoum\\mock\\generator\\method\\exception' => $vendorDir . '/atoum/atoum/classes/mock/generator/method/exception.php',
    'mageekguy\\atoum\\mock\\php\\method' => $vendorDir . '/atoum/atoum/classes/mock/php/method.php',
    'mageekguy\\atoum\\mock\\php\\method\\argument' => $vendorDir . '/atoum/atoum/classes/mock/php/method/argument.php',
    'mageekguy\\atoum\\mock\\stream' => $vendorDir . '/atoum/atoum/classes/mock/stream.php',
    'mageekguy\\atoum\\mock\\stream\\controller' => $vendorDir . '/atoum/atoum/classes/mock/stream/controller.php',
    'mageekguy\\atoum\\mock\\stream\\invoker' => $vendorDir . '/atoum/atoum/classes/mock/stream/invoker.php',
    'mageekguy\\atoum\\mock\\streams\\fs\\controller' => $vendorDir . '/atoum/atoum/classes/mock/streams/fs/controller.php',
    'mageekguy\\atoum\\mock\\streams\\fs\\controller\\factory' => $vendorDir . '/atoum/atoum/classes/mock/streams/fs/controller/factory.php',
    'mageekguy\\atoum\\mock\\streams\\fs\\directory' => $vendorDir . '/atoum/atoum/classes/mock/streams/fs/directory.php',
    'mageekguy\\atoum\\mock\\streams\\fs\\directory\\controller' => $vendorDir . '/atoum/atoum/classes/mock/streams/fs/directory/controller.php',
    'mageekguy\\atoum\\mock\\streams\\fs\\file' => $vendorDir . '/atoum/atoum/classes/mock/streams/fs/file.php',
    'mageekguy\\atoum\\mock\\streams\\fs\\file\\controller' => $vendorDir . '/atoum/atoum/classes/mock/streams/fs/file/controller.php',
    'mageekguy\\atoum\\observable' => $vendorDir . '/atoum/atoum/classes/observable.php',
    'mageekguy\\atoum\\observer' => $vendorDir . '/atoum/atoum/classes/observer.php',
    'mageekguy\\atoum\\observers\\runner' => $vendorDir . '/atoum/atoum/classes/observers/runner.php',
    'mageekguy\\atoum\\observers\\test' => $vendorDir . '/atoum/atoum/classes/observers/test.php',
    'mageekguy\\atoum\\php' => $vendorDir . '/atoum/atoum/classes/php.php',
    'mageekguy\\atoum\\php\\call' => $vendorDir . '/atoum/atoum/classes/php/call.php',
    'mageekguy\\atoum\\php\\exception' => $vendorDir . '/atoum/atoum/classes/php/exception.php',
    'mageekguy\\atoum\\php\\extension' => $vendorDir . '/atoum/atoum/classes/php/extension.php',
    'mageekguy\\atoum\\php\\mocker' => $vendorDir . '/atoum/atoum/classes/php/mocker.php',
    'mageekguy\\atoum\\php\\mocker\\adapter' => $vendorDir . '/atoum/atoum/classes/php/mocker/adapter.php',
    'mageekguy\\atoum\\php\\mocker\\adapter\\invoker' => $vendorDir . '/atoum/atoum/classes/php/mocker/adapter/invoker.php',
    'mageekguy\\atoum\\php\\mocker\\constant' => $vendorDir . '/atoum/atoum/classes/php/mocker/constant.php',
    'mageekguy\\atoum\\php\\mocker\\exception' => $vendorDir . '/atoum/atoum/classes/php/mocker/exception.php',
    'mageekguy\\atoum\\php\\mocker\\exceptions\\constant' => $vendorDir . '/atoum/atoum/classes/php/mocker/exceptions/constant.php',
    'mageekguy\\atoum\\php\\mocker\\funktion' => $vendorDir . '/atoum/atoum/classes/php/mocker/funktion.php',
    'mageekguy\\atoum\\php\\tokenizer' => $vendorDir . '/atoum/atoum/classes/php/tokenizer.php',
    'mageekguy\\atoum\\php\\tokenizer\\iterator' => $vendorDir . '/atoum/atoum/classes/php/tokenizer/iterator.php',
    'mageekguy\\atoum\\php\\tokenizer\\iterator\\value' => $vendorDir . '/atoum/atoum/classes/php/tokenizer/iterator/value.php',
    'mageekguy\\atoum\\php\\tokenizer\\iterators\\phpArgument' => $vendorDir . '/atoum/atoum/classes/php/tokenizer/iterators/phpArgument.php',
    'mageekguy\\atoum\\php\\tokenizer\\iterators\\phpClass' => $vendorDir . '/atoum/atoum/classes/php/tokenizer/iterators/phpClass.php',
    'mageekguy\\atoum\\php\\tokenizer\\iterators\\phpConstant' => $vendorDir . '/atoum/atoum/classes/php/tokenizer/iterators/phpConstant.php',
    'mageekguy\\atoum\\php\\tokenizer\\iterators\\phpDefaultValue' => $vendorDir . '/atoum/atoum/classes/php/tokenizer/iterators/phpDefaultValue.php',
    'mageekguy\\atoum\\php\\tokenizer\\iterators\\phpFunction' => $vendorDir . '/atoum/atoum/classes/php/tokenizer/iterators/phpFunction.php',
    'mageekguy\\atoum\\php\\tokenizer\\iterators\\phpImportation' => $vendorDir . '/atoum/atoum/classes/php/tokenizer/iterators/phpImportation.php',
    'mageekguy\\atoum\\php\\tokenizer\\iterators\\phpMethod' => $vendorDir . '/atoum/atoum/classes/php/tokenizer/iterators/phpMethod.php',
    'mageekguy\\atoum\\php\\tokenizer\\iterators\\phpNamespace' => $vendorDir . '/atoum/atoum/classes/php/tokenizer/iterators/phpNamespace.php',
    'mageekguy\\atoum\\php\\tokenizer\\iterators\\phpProperty' => $vendorDir . '/atoum/atoum/classes/php/tokenizer/iterators/phpProperty.php',
    'mageekguy\\atoum\\php\\tokenizer\\iterators\\phpScript' => $vendorDir . '/atoum/atoum/classes/php/tokenizer/iterators/phpScript.php',
    'mageekguy\\atoum\\php\\tokenizer\\token' => $vendorDir . '/atoum/atoum/classes/php/tokenizer/token.php',
    'mageekguy\\atoum\\reader' => $vendorDir . '/atoum/atoum/classes/reader.php',
    'mageekguy\\atoum\\readers\\std\\in' => $vendorDir . '/atoum/atoum/classes/readers/std/in.php',
    'mageekguy\\atoum\\report' => $vendorDir . '/atoum/atoum/classes/report.php',
    'mageekguy\\atoum\\report\\field' => $vendorDir . '/atoum/atoum/classes/report/field.php',
    'mageekguy\\atoum\\report\\fields\\event' => $vendorDir . '/atoum/atoum/classes/report/fields/event.php',
    'mageekguy\\atoum\\report\\fields\\runner' => $vendorDir . '/atoum/atoum/classes/report/fields/runner.php',
    'mageekguy\\atoum\\report\\fields\\runner\\atoum' => $vendorDir . '/atoum/atoum/classes/report/fields/runner/atoum.php',
    'mageekguy\\atoum\\report\\fields\\runner\\atoum\\cli' => $vendorDir . '/atoum/atoum/classes/report/fields/runner/atoum/cli.php',
    'mageekguy\\atoum\\report\\fields\\runner\\atoum\\logo' => $vendorDir . '/atoum/atoum/classes/report/fields/runner/atoum/logo.php',
    'mageekguy\\atoum\\report\\fields\\runner\\atoum\\path' => $vendorDir . '/atoum/atoum/classes/report/fields/runner/atoum/path.php',
    'mageekguy\\atoum\\report\\fields\\runner\\atoum\\path\\cli' => $vendorDir . '/atoum/atoum/classes/report/fields/runner/atoum/path/cli.php',
    'mageekguy\\atoum\\report\\fields\\runner\\atoum\\phing' => $vendorDir . '/atoum/atoum/classes/report/fields/runner/atoum/phing.php',
    'mageekguy\\atoum\\report\\fields\\runner\\atoum\\version' => $vendorDir . '/atoum/atoum/classes/report/fields/runner/atoum/version.php',
    'mageekguy\\atoum\\report\\fields\\runner\\atoum\\version\\cli' => $vendorDir . '/atoum/atoum/classes/report/fields/runner/atoum/version/cli.php',
    'mageekguy\\atoum\\report\\fields\\runner\\coverage' => $vendorDir . '/atoum/atoum/classes/report/fields/runner/coverage.php',
    'mageekguy\\atoum\\report\\fields\\runner\\coverage\\cli' => $vendorDir . '/atoum/atoum/classes/report/fields/runner/coverage/cli.php',
    'mageekguy\\atoum\\report\\fields\\runner\\coverage\\html' => $vendorDir . '/atoum/atoum/classes/report/fields/runner/coverage/html.php',
    'mageekguy\\atoum\\report\\fields\\runner\\coverage\\treemap' => $vendorDir . '/atoum/atoum/classes/report/fields/runner/coverage/treemap.php',
    'mageekguy\\atoum\\report\\fields\\runner\\duration' => $vendorDir . '/atoum/atoum/classes/report/fields/runner/duration.php',
    'mageekguy\\atoum\\report\\fields\\runner\\duration\\cli' => $vendorDir . '/atoum/atoum/classes/report/fields/runner/duration/cli.php',
    'mageekguy\\atoum\\report\\fields\\runner\\duration\\phing' => $vendorDir . '/atoum/atoum/classes/report/fields/runner/duration/phing.php',
    'mageekguy\\atoum\\report\\fields\\runner\\errors' => $vendorDir . '/atoum/atoum/classes/report/fields/runner/errors.php',
    'mageekguy\\atoum\\report\\fields\\runner\\errors\\cli' => $vendorDir . '/atoum/atoum/classes/report/fields/runner/errors/cli.php',
    'mageekguy\\atoum\\report\\fields\\runner\\event' => $vendorDir . '/atoum/atoum/classes/report/fields/runner/event.php',
    'mageekguy\\atoum\\report\\fields\\runner\\event\\cli' => $vendorDir . '/atoum/atoum/classes/report/fields/runner/event/cli.php',
    'mageekguy\\atoum\\report\\fields\\runner\\event\\nyancat' => $vendorDir . '/atoum/atoum/classes/report/fields/runner/event/nyancat.php',
    'mageekguy\\atoum\\report\\fields\\runner\\event\\santa' => $vendorDir . '/atoum/atoum/classes/report/fields/runner/event/santa.php',
    'mageekguy\\atoum\\report\\fields\\runner\\exceptions' => $vendorDir . '/atoum/atoum/classes/report/fields/runner/exceptions.php',
    'mageekguy\\atoum\\report\\fields\\runner\\exceptions\\cli' => $vendorDir . '/atoum/atoum/classes/report/fields/runner/exceptions/cli.php',
    'mageekguy\\atoum\\report\\fields\\runner\\failures' => $vendorDir . '/atoum/atoum/classes/report/fields/runner/failures.php',
    'mageekguy\\atoum\\report\\fields\\runner\\failures\\cli' => $vendorDir . '/atoum/atoum/classes/report/fields/runner/failures/cli.php',
    'mageekguy\\atoum\\report\\fields\\runner\\failures\\execute' => $vendorDir . '/atoum/atoum/classes/report/fields/runner/failures/execute.php',
    'mageekguy\\atoum\\report\\fields\\runner\\failures\\execute\\macos\\macvim' => $vendorDir . '/atoum/atoum/classes/report/fields/runner/failures/execute/macos/macvim.php',
    'mageekguy\\atoum\\report\\fields\\runner\\failures\\execute\\macos\\phpstorm' => $vendorDir . '/atoum/atoum/classes/report/fields/runner/failures/execute/macos/phpstorm.php',
    'mageekguy\\atoum\\report\\fields\\runner\\failures\\execute\\unix\\gedit' => $vendorDir . '/atoum/atoum/classes/report/fields/runner/failures/execute/unix/gedit.php',
    'mageekguy\\atoum\\report\\fields\\runner\\failures\\execute\\unix\\gvim' => $vendorDir . '/atoum/atoum/classes/report/fields/runner/failures/execute/unix/gvim.php',
    'mageekguy\\atoum\\report\\fields\\runner\\failures\\execute\\unix\\phpstorm' => $vendorDir . '/atoum/atoum/classes/report/fields/runner/failures/execute/unix/phpstorm.php',
    'mageekguy\\atoum\\report\\fields\\runner\\outputs' => $vendorDir . '/atoum/atoum/classes/report/fields/runner/outputs.php',
    'mageekguy\\atoum\\report\\fields\\runner\\outputs\\cli' => $vendorDir . '/atoum/atoum/classes/report/fields/runner/outputs/cli.php',
    'mageekguy\\atoum\\report\\fields\\runner\\php\\path' => $vendorDir . '/atoum/atoum/classes/report/fields/runner/php/path.php',
    'mageekguy\\atoum\\report\\fields\\runner\\php\\path\\cli' => $vendorDir . '/atoum/atoum/classes/report/fields/runner/php/path/cli.php',
    'mageekguy\\atoum\\report\\fields\\runner\\php\\version' => $vendorDir . '/atoum/atoum/classes/report/fields/runner/php/version.php',
    'mageekguy\\atoum\\report\\fields\\runner\\php\\version\\cli' => $vendorDir . '/atoum/atoum/classes/report/fields/runner/php/version/cli.php',
    'mageekguy\\atoum\\report\\fields\\runner\\result' => $vendorDir . '/atoum/atoum/classes/report/fields/runner/result.php',
    'mageekguy\\atoum\\report\\fields\\runner\\result\\cli' => $vendorDir . '/atoum/atoum/classes/report/fields/runner/result/cli.php',
    'mageekguy\\atoum\\report\\fields\\runner\\result\\logo' => $vendorDir . '/atoum/atoum/classes/report/fields/runner/result/logo.php',
    'mageekguy\\atoum\\report\\fields\\runner\\result\\notifier' => $vendorDir . '/atoum/atoum/classes/report/fields/runner/result/notifier.php',
    'mageekguy\\atoum\\report\\fields\\runner\\result\\notifier\\image' => $vendorDir . '/atoum/atoum/classes/report/fields/runner/result/notifier/image.php',
    'mageekguy\\atoum\\report\\fields\\runner\\result\\notifier\\image\\growl' => $vendorDir . '/atoum/atoum/classes/report/fields/runner/result/notifier/image/growl.php',
    'mageekguy\\atoum\\report\\fields\\runner\\result\\notifier\\image\\libnotify' => $vendorDir . '/atoum/atoum/classes/report/fields/runner/result/notifier/image/libnotify.php',
    'mageekguy\\atoum\\report\\fields\\runner\\result\\notifier\\terminal' => $vendorDir . '/atoum/atoum/classes/report/fields/runner/result/notifier/terminal.php',
    'mageekguy\\atoum\\report\\fields\\runner\\tap\\plan' => $vendorDir . '/atoum/atoum/classes/report/fields/runner/tap/plan.php',
    'mageekguy\\atoum\\report\\fields\\runner\\tests\\coverage' => $vendorDir . '/atoum/atoum/classes/report/fields/runner/tests/coverage.php',
    'mageekguy\\atoum\\report\\fields\\runner\\tests\\coverage\\cli' => $vendorDir . '/atoum/atoum/classes/report/fields/runner/tests/coverage/cli.php',
    'mageekguy\\atoum\\report\\fields\\runner\\tests\\coverage\\phing' => $vendorDir . '/atoum/atoum/classes/report/fields/runner/tests/coverage/phing.php',
    'mageekguy\\atoum\\report\\fields\\runner\\tests\\duration' => $vendorDir . '/atoum/atoum/classes/report/fields/runner/tests/duration.php',
    'mageekguy\\atoum\\report\\fields\\runner\\tests\\duration\\cli' => $vendorDir . '/atoum/atoum/classes/report/fields/runner/tests/duration/cli.php',
    'mageekguy\\atoum\\report\\fields\\runner\\tests\\memory' => $vendorDir . '/atoum/atoum/classes/report/fields/runner/tests/memory.php',
    'mageekguy\\atoum\\report\\fields\\runner\\tests\\memory\\cli' => $vendorDir . '/atoum/atoum/classes/report/fields/runner/tests/memory/cli.php',
    'mageekguy\\atoum\\report\\fields\\runner\\tests\\memory\\phing' => $vendorDir . '/atoum/atoum/classes/report/fields/runner/tests/memory/phing.php',
    'mageekguy\\atoum\\report\\fields\\runner\\tests\\skipped' => $vendorDir . '/atoum/atoum/classes/report/fields/runner/tests/skipped.php',
    'mageekguy\\atoum\\report\\fields\\runner\\tests\\skipped\\cli' => $vendorDir . '/atoum/atoum/classes/report/fields/runner/tests/skipped/cli.php',
    'mageekguy\\atoum\\report\\fields\\runner\\tests\\uncompleted' => $vendorDir . '/atoum/atoum/classes/report/fields/runner/tests/uncompleted.php',
    'mageekguy\\atoum\\report\\fields\\runner\\tests\\uncompleted\\cli' => $vendorDir . '/atoum/atoum/classes/report/fields/runner/tests/uncompleted/cli.php',
    'mageekguy\\atoum\\report\\fields\\runner\\tests\\void' => $vendorDir . '/atoum/atoum/classes/report/fields/runner/tests/void.php',
    'mageekguy\\atoum\\report\\fields\\runner\\tests\\void\\cli' => $vendorDir . '/atoum/atoum/classes/report/fields/runner/tests/void/cli.php',
    'mageekguy\\atoum\\report\\fields\\test\\duration' => $vendorDir . '/atoum/atoum/classes/report/fields/test/duration.php',
    'mageekguy\\atoum\\report\\fields\\test\\duration\\cli' => $vendorDir . '/atoum/atoum/classes/report/fields/test/duration/cli.php',
    'mageekguy\\atoum\\report\\fields\\test\\duration\\phing' => $vendorDir . '/atoum/atoum/classes/report/fields/test/duration/phing.php',
    'mageekguy\\atoum\\report\\fields\\test\\event' => $vendorDir . '/atoum/atoum/classes/report/fields/test/event.php',
    'mageekguy\\atoum\\report\\fields\\test\\event\\cli' => $vendorDir . '/atoum/atoum/classes/report/fields/test/event/cli.php',
    'mageekguy\\atoum\\report\\fields\\test\\event\\phing' => $vendorDir . '/atoum/atoum/classes/report/fields/test/event/phing.php',
    'mageekguy\\atoum\\report\\fields\\test\\event\\tap' => $vendorDir . '/atoum/atoum/classes/report/fields/test/event/tap.php',
    'mageekguy\\atoum\\report\\fields\\test\\memory' => $vendorDir . '/atoum/atoum/classes/report/fields/test/memory.php',
    'mageekguy\\atoum\\report\\fields\\test\\memory\\cli' => $vendorDir . '/atoum/atoum/classes/report/fields/test/memory/cli.php',
    'mageekguy\\atoum\\report\\fields\\test\\memory\\phing' => $vendorDir . '/atoum/atoum/classes/report/fields/test/memory/phing.php',
    'mageekguy\\atoum\\report\\fields\\test\\run' => $vendorDir . '/atoum/atoum/classes/report/fields/test/run.php',
    'mageekguy\\atoum\\report\\fields\\test\\run\\cli' => $vendorDir . '/atoum/atoum/classes/report/fields/test/run/cli.php',
    'mageekguy\\atoum\\report\\fields\\test\\run\\phing' => $vendorDir . '/atoum/atoum/classes/report/fields/test/run/phing.php',
    'mageekguy\\atoum\\report\\writers\\asynchronous' => $vendorDir . '/atoum/atoum/classes/report/writers/asynchronous.php',
    'mageekguy\\atoum\\report\\writers\\realtime' => $vendorDir . '/atoum/atoum/classes/report/writers/realtime.php',
    'mageekguy\\atoum\\reports\\asynchronous' => $vendorDir . '/atoum/atoum/classes/reports/asynchronous.php',
    'mageekguy\\atoum\\reports\\asynchronous\\builder' => $vendorDir . '/atoum/atoum/classes/reports/asynchronous/builder.php',
    'mageekguy\\atoum\\reports\\asynchronous\\clover' => $vendorDir . '/atoum/atoum/classes/reports/asynchronous/clover.php',
    'mageekguy\\atoum\\reports\\asynchronous\\coveralls' => $vendorDir . '/atoum/atoum/classes/reports/asynchronous/coveralls.php',
    'mageekguy\\atoum\\reports\\asynchronous\\vim' => $vendorDir . '/atoum/atoum/classes/reports/asynchronous/vim.php',
    'mageekguy\\atoum\\reports\\asynchronous\\xunit' => $vendorDir . '/atoum/atoum/classes/reports/asynchronous/xunit.php',
    'mageekguy\\atoum\\reports\\realtime' => $vendorDir . '/atoum/atoum/classes/reports/realtime.php',
    'mageekguy\\atoum\\reports\\realtime\\cli' => $vendorDir . '/atoum/atoum/classes/reports/realtime/cli.php',
    'mageekguy\\atoum\\reports\\realtime\\cli\\light' => $vendorDir . '/atoum/atoum/classes/reports/realtime/cli/light.php',
    'mageekguy\\atoum\\reports\\realtime\\nyancat' => $vendorDir . '/atoum/atoum/classes/reports/realtime/nyancat.php',
    'mageekguy\\atoum\\reports\\realtime\\phing' => $vendorDir . '/atoum/atoum/classes/reports/realtime/phing.php',
    'mageekguy\\atoum\\reports\\realtime\\santa' => $vendorDir . '/atoum/atoum/classes/reports/realtime/santa.php',
    'mageekguy\\atoum\\reports\\realtime\\tap' => $vendorDir . '/atoum/atoum/classes/reports/realtime/tap.php',
    'mageekguy\\atoum\\runner' => $vendorDir . '/atoum/atoum/classes/runner.php',
    'mageekguy\\atoum\\runner\\score' => $vendorDir . '/atoum/atoum/classes/runner/score.php',
    'mageekguy\\atoum\\score' => $vendorDir . '/atoum/atoum/classes/score.php',
    'mageekguy\\atoum\\score\\coverage' => $vendorDir . '/atoum/atoum/classes/score/coverage.php',
    'mageekguy\\atoum\\script' => $vendorDir . '/atoum/atoum/classes/script.php',
    'mageekguy\\atoum\\script\\arguments\\parser' => $vendorDir . '/atoum/atoum/classes/script/arguments/parser.php',
    'mageekguy\\atoum\\script\\configurable' => $vendorDir . '/atoum/atoum/classes/script/configurable.php',
    'mageekguy\\atoum\\script\\prompt' => $vendorDir . '/atoum/atoum/classes/script/prompt.php',
    'mageekguy\\atoum\\scripts\\builder' => $vendorDir . '/atoum/atoum/classes/scripts/builder.php',
    'mageekguy\\atoum\\scripts\\builder\\vcs' => $vendorDir . '/atoum/atoum/classes/scripts/builder/vcs.php',
    'mageekguy\\atoum\\scripts\\builder\\vcs\\svn' => $vendorDir . '/atoum/atoum/classes/scripts/builder/vcs/svn.php',
    'mageekguy\\atoum\\scripts\\compiler' => $vendorDir . '/atoum/atoum/classes/scripts/compiler.php',
    'mageekguy\\atoum\\scripts\\coverage' => $vendorDir . '/atoum/atoum/classes/scripts/coverage.php',
    'mageekguy\\atoum\\scripts\\git\\pusher' => $vendorDir . '/atoum/atoum/classes/scripts/git/pusher.php',
    'mageekguy\\atoum\\scripts\\phar\\generator' => $vendorDir . '/atoum/atoum/classes/scripts/phar/generator.php',
    'mageekguy\\atoum\\scripts\\phar\\stub' => $vendorDir . '/atoum/atoum/classes/scripts/phar/stub.php',
    'mageekguy\\atoum\\scripts\\runner' => $vendorDir . '/atoum/atoum/classes/scripts/runner.php',
    'mageekguy\\atoum\\scripts\\runner\\looper' => $vendorDir . '/atoum/atoum/classes/scripts/runner/looper.php',
    'mageekguy\\atoum\\scripts\\runner\\loopers\\prompt' => $vendorDir . '/atoum/atoum/classes/scripts/runner/loopers/prompt.php',
    'mageekguy\\atoum\\scripts\\tagger' => $vendorDir . '/atoum/atoum/classes/scripts/tagger.php',
    'mageekguy\\atoum\\scripts\\tagger\\engine' => $vendorDir . '/atoum/atoum/classes/scripts/tagger/engine.php',
    'mageekguy\\atoum\\scripts\\treemap' => $vendorDir . '/atoum/atoum/classes/scripts/treemap.php',
    'mageekguy\\atoum\\scripts\\treemap\\analyzer' => $vendorDir . '/atoum/atoum/classes/scripts/treemap/analyzer.php',
    'mageekguy\\atoum\\scripts\\treemap\\analyzer\\generic' => $vendorDir . '/atoum/atoum/classes/scripts/treemap/analyzer/generic.php',
    'mageekguy\\atoum\\scripts\\treemap\\analyzers\\size' => $vendorDir . '/atoum/atoum/classes/scripts/treemap/analyzers/size.php',
    'mageekguy\\atoum\\scripts\\treemap\\analyzers\\sloc' => $vendorDir . '/atoum/atoum/classes/scripts/treemap/analyzers/sloc.php',
    'mageekguy\\atoum\\scripts\\treemap\\analyzers\\token' => $vendorDir . '/atoum/atoum/classes/scripts/treemap/analyzers/token.php',
    'mageekguy\\atoum\\scripts\\treemap\\categorizer' => $vendorDir . '/atoum/atoum/classes/scripts/treemap/categorizer.php',
    'mageekguy\\atoum\\superglobals' => $vendorDir . '/atoum/atoum/classes/superglobals.php',
    'mageekguy\\atoum\\template' => $vendorDir . '/atoum/atoum/classes/template.php',
    'mageekguy\\atoum\\template\\data' => $vendorDir . '/atoum/atoum/classes/template/data.php',
    'mageekguy\\atoum\\template\\iterator' => $vendorDir . '/atoum/atoum/classes/template/iterator.php',
    'mageekguy\\atoum\\template\\parser' => $vendorDir . '/atoum/atoum/classes/template/parser.php',
    'mageekguy\\atoum\\template\\parser\\exception' => $vendorDir . '/atoum/atoum/classes/template/parser/exception.php',
    'mageekguy\\atoum\\template\\tag' => $vendorDir . '/atoum/atoum/classes/template/tag.php',
    'mageekguy\\atoum\\test' => $vendorDir . '/atoum/atoum/classes/test.php',
    'mageekguy\\atoum\\test\\adapter' => $vendorDir . '/atoum/atoum/classes/test/adapter.php',
    'mageekguy\\atoum\\test\\adapter\\call' => $vendorDir . '/atoum/atoum/classes/test/adapter/call.php',
    'mageekguy\\atoum\\test\\adapter\\call\\arguments\\decorator' => $vendorDir . '/atoum/atoum/classes/test/adapter/call/arguments/decorator.php',
    'mageekguy\\atoum\\test\\adapter\\call\\decorator' => $vendorDir . '/atoum/atoum/classes/test/adapter/call/decorator.php',
    'mageekguy\\atoum\\test\\adapter\\call\\decorators\\addClass' => $vendorDir . '/atoum/atoum/classes/test/adapter/call/decorators/addClass.php',
    'mageekguy\\atoum\\test\\adapter\\calls' => $vendorDir . '/atoum/atoum/classes/test/adapter/calls.php',
    'mageekguy\\atoum\\test\\adapter\\calls\\decorator' => $vendorDir . '/atoum/atoum/classes/test/adapter/calls/decorator.php',
    'mageekguy\\atoum\\test\\adapter\\invoker' => $vendorDir . '/atoum/atoum/classes/test/adapter/invoker.php',
    'mageekguy\\atoum\\test\\adapter\\storage' => $vendorDir . '/atoum/atoum/classes/test/adapter/storage.php',
    'mageekguy\\atoum\\test\\asserter\\generator' => $vendorDir . '/atoum/atoum/classes/test/asserter/generator.php',
    'mageekguy\\atoum\\test\\assertion\\aliaser' => $vendorDir . '/atoum/atoum/classes/test/assertion/aliaser.php',
    'mageekguy\\atoum\\test\\assertion\\manager' => $vendorDir . '/atoum/atoum/classes/test/assertion/manager.php',
    'mageekguy\\atoum\\test\\assertion\\manager\\exception' => $vendorDir . '/atoum/atoum/classes/test/assertion/manager/exception.php',
    'mageekguy\\atoum\\test\\data\\provider' => $vendorDir . '/atoum/atoum/classes/test/data/provider.php',
    'mageekguy\\atoum\\test\\data\\provider\\aggregator' => $vendorDir . '/atoum/atoum/classes/test/data/provider/aggregator.php',
    'mageekguy\\atoum\\test\\data\\provider\\object\\exceptions\\mandatoryArgument' => $vendorDir . '/atoum/atoum/classes/test/data/provider/object/exceptions/mandatoryArgument.php',
    'mageekguy\\atoum\\test\\data\\provider\\object\\exceptions\\privateConstructor' => $vendorDir . '/atoum/atoum/classes/test/data/provider/object/exceptions/privateConstructor.php',
    'mageekguy\\atoum\\test\\data\\providers\\mock' => $vendorDir . '/atoum/atoum/classes/test/data/providers/mock.php',
    'mageekguy\\atoum\\test\\data\\providers\\object' => $vendorDir . '/atoum/atoum/classes/test/data/providers/object.php',
    'mageekguy\\atoum\\test\\data\\set' => $vendorDir . '/atoum/atoum/classes/test/data/set.php',
    'mageekguy\\atoum\\test\\engine' => $vendorDir . '/atoum/atoum/classes/test/engine.php',
    'mageekguy\\atoum\\test\\engines\\concurrent' => $vendorDir . '/atoum/atoum/classes/test/engines/concurrent.php',
    'mageekguy\\atoum\\test\\engines\\inline' => $vendorDir . '/atoum/atoum/classes/test/engines/inline.php',
    'mageekguy\\atoum\\test\\engines\\isolate' => $vendorDir . '/atoum/atoum/classes/test/engines/isolate.php',
    'mageekguy\\atoum\\test\\exceptions\\runtime' => $vendorDir . '/atoum/atoum/classes/test/exceptions/runtime.php',
    'mageekguy\\atoum\\test\\exceptions\\skip' => $vendorDir . '/atoum/atoum/classes/test/exceptions/skip.php',
    'mageekguy\\atoum\\test\\exceptions\\stop' => $vendorDir . '/atoum/atoum/classes/test/exceptions/stop.php',
    'mageekguy\\atoum\\test\\generator' => $vendorDir . '/atoum/atoum/classes/test/generator.php',
    'mageekguy\\atoum\\test\\generator\\exception' => $vendorDir . '/atoum/atoum/classes/test/generator/exception.php',
    'mageekguy\\atoum\\test\\mock\\generator' => $vendorDir . '/atoum/atoum/classes/test/mock/generator.php',
    'mageekguy\\atoum\\test\\score' => $vendorDir . '/atoum/atoum/classes/test/score.php',
    'mageekguy\\atoum\\tools\\diff' => $vendorDir . '/atoum/atoum/classes/tools/diff.php',
    'mageekguy\\atoum\\tools\\diff\\decorator' => $vendorDir . '/atoum/atoum/classes/tools/diff/decorator.php',
    'mageekguy\\atoum\\tools\\diffs\\variable' => $vendorDir . '/atoum/atoum/classes/tools/diffs/variable.php',
    'mageekguy\\atoum\\tools\\variable\\analyzer' => $vendorDir . '/atoum/atoum/classes/tools/variable/analyzer.php',
    'mageekguy\\atoum\\writer' => $vendorDir . '/atoum/atoum/classes/writer.php',
    'mageekguy\\atoum\\writer\\decorator' => $vendorDir . '/atoum/atoum/classes/writer/decorator.php',
    'mageekguy\\atoum\\writer\\decorators\\eol' => $vendorDir . '/atoum/atoum/classes/writer/decorators/eol.php',
    'mageekguy\\atoum\\writer\\decorators\\prompt' => $vendorDir . '/atoum/atoum/classes/writer/decorators/prompt.php',
    'mageekguy\\atoum\\writer\\decorators\\rtrim' => $vendorDir . '/atoum/atoum/classes/writer/decorators/rtrim.php',
    'mageekguy\\atoum\\writer\\decorators\\trim' => $vendorDir . '/atoum/atoum/classes/writer/decorators/trim.php',
    'mageekguy\\atoum\\writers\\file' => $vendorDir . '/atoum/atoum/classes/writers/file.php',
    'mageekguy\\atoum\\writers\\http' => $vendorDir . '/atoum/atoum/classes/writers/http.php',
    'mageekguy\\atoum\\writers\\http\\exception' => $vendorDir . '/atoum/atoum/classes/writers/http/exception.php',
    'mageekguy\\atoum\\writers\\mail' => $vendorDir . '/atoum/atoum/classes/writers/mail.php',
    'mageekguy\\atoum\\writers\\std' => $vendorDir . '/atoum/atoum/classes/writers/std.php',
    'mageekguy\\atoum\\writers\\std\\err' => $vendorDir . '/atoum/atoum/classes/writers/std/err.php',
    'mageekguy\\atoum\\writers\\std\\out' => $vendorDir . '/atoum/atoum/classes/writers/std/out.php',
);
